﻿using Microsoft.IdentityModel.Tokens;
using SushiBarContracts.BindingModel;
using SushiBarContracts.SearchModel;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SushiBarDatabaseImplement.Implements
{
    public class ImplementerStorage : IImplementerStorage
    {
        public ImplementerViewModel? Insert(ImplementerBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var newImplementer = Implementer.Create(model);
            if (newImplementer == null) { return null; }
            context.Implementers.Add(newImplementer);
            context.SaveChanges();
            return newImplementer.GetViewModel;
        }

        public ImplementerViewModel? Update(ImplementerBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var implementer = context.Implementers.FirstOrDefault(x => x.Id == model.Id);
            if (implementer == null) { return null; };
            implementer.Update(model);
            context.SaveChanges();
            return implementer.GetViewModel;
        }

        public ImplementerViewModel? Delete(ImplementerBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var element = context.Implementers.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Implementers.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public ImplementerViewModel? GetElement(ImplementerSearchModel model)
        {
            return GetFilteredList(model).FirstOrDefault();
        }

        public List<ImplementerViewModel> GetFilteredList(ImplementerSearchModel model)
        {
            using var context = new SushiBarDatabase();
            var implementers = context.Implementers.Select(x => x.GetViewModel).ToList();
            if (model.Id.HasValue)
            {
                implementers = implementers.Where(x => x.Id == model.Id.Value).ToList();
            }
            if (!model.ImplementerFIO.IsNullOrEmpty())
            {
                implementers = implementers.Where(x => x.ImplementerFIO == model.ImplementerFIO).ToList();
            }
            if (!model.Password.IsNullOrEmpty())
            {
                implementers = implementers.Where(x => x.Password == model.Password).ToList();
            }
            return implementers;
        }

        public List<ImplementerViewModel> GetFullList()
        {
            using var context = new SushiBarDatabase();
            return context.Implementers.Select(x => x.GetViewModel).ToList();
        }
    }
}
